@@ -1,20 +1,24 @@
 # Copyright (C) 2007 Collabora Ltd. <http://www.collabora.co.uk/>
 #
-# Licensed under the Academic Free License version 2.1
+# Permission is hereby granted, free of charge, to any person
+# obtaining a copy of this software and associated documentation
+# files (the "Software"), to deal in the Software without
+# restriction, including without limitation the rights to use, copy,
+# modify, merge, publish, distribute, sublicense, and/or sell copies
+# of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published
-# by the Free Software Foundation; either version 2.1 of the License, or
-# (at your option) any later version.
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 
 __all__ = ('Connection', 'SignalMatch')
 __docformat__ = 'reStructuredText'
@@ -242,6 +246,8 @@ class Connection(_Connection):
         if not hasattr(self, '_dbus_Connection_initialized'):
             self._dbus_Connection_initialized = 1
 
+            self.__call_on_disconnection = []
+
             self._signal_recipients_by_object_path = {}
             """Map from object path to dict mapping dbus_interface to dict
             mapping member to list of SignalMatch objects."""
@@ -504,6 +510,19 @@ class Connection(_Connection):
         for match in self._iter_easy_matches(path, dbus_interface,
                                              signal_name):
             match.maybe_handle_message(message)
+
+        if (dbus_interface == LOCAL_IFACE and
+            path == LOCAL_PATH and
+            signal_name == 'Disconnected'):
+            for cb in self.__call_on_disconnection:
+                try:
+                    cb(self)
+                except Exception, e:
+                    # basicConfig is a no-op if logging is already configured
+                    logging.basicConfig()
+                    _logger.error('Exception in handler for Disconnected '
+                        'signal:', exc_info=1)
+
         return HANDLER_RESULT_NOT_YET_HANDLED
 
     def call_async(self, bus_name, object_path, dbus_interface, method,
@@ -608,3 +627,12 @@ class Connection(_Connection):
             return args_list[0]
         else:
             return tuple(args_list)
+
+    def call_on_disconnection(self, callable):
+        """Arrange for `callable` to be called with one argument (this
+        Connection object) when the Connection becomes
+        disconnected.
+
+        :Since: 0.83.0
+        """
+        self.__call_on_disconnection.append(callable)
